from collections import Counter
from spacy.language import Language
from spacy.tokens import Doc

@Language.component("formality")
def formality(doc):
    # Add the extension to Doc
    Doc.set_extension("formality", default=0, force=True)

    # get counts for each POS for words (is_alpha is true)
    counts = Counter([ token.pos_ for token in doc if token.is_alpha ])

    # get the total number of tokens
    total = sum(counts[c] for c in counts)

    # only calc formality if 200 tokens or more
    if total < 200:
        #print(f"Less than 200 tokens found ({total})")
        doc._.formality = None
    else:
        # calculate frequencies
        # non-deictic
        noun_freq   = (counts["NOUN"] / total) * 100 if "NOUN" in counts else 0
        prop_freq   = (counts["PROPN"] / total) * 100 if "PROPN" in counts else 0
        adj_freq    = (counts["ADJ"] / total) * 100 if "ADJ" in counts else 0
        prep_freq   = (counts["ADP"] / total) * 100 if "ADP" in counts else 0
        # just want articles - Spacy lumps these in DET so need to do text matching
        article_freq = (len([token for token in doc if token.text in ["the","a","an"]]) / total) * 100
        # deictic
        pron_freq   = (counts["PRON"] / total) * 100 if "PRON" in counts else 0
        verb_freq   = (counts["VERB"] / total) * 100 if "VERB" in counts else 0
        aux_freq    = (counts["AUX"] / total) * 100 if "AUX" in counts else 0 # AUX is usually a verb so include in deictic
        adverb_freq = (counts["ADV"] / total) * 100 if "ADV" in counts else 0
        intj_freq   = (counts["INTJ"] / total) * 100 if "INTJ" in counts else 0

        # HEYLIGHEN & DEWAELE's formula:
        # F = (noun frequency + adjective freq. + preposition freq. + article freq. – pronoun freq.
        # – verb freq. – adverb freq. – interjection freq. + 100)/2
        # Gives a value between 0 (least formal) and 100 (most formal)
        n = noun_freq + prop_freq + adj_freq + prep_freq + article_freq
        c = pron_freq + verb_freq + aux_freq + adverb_freq + intj_freq
        f = (n - c + 100)/2

        # add the score to the doc (rounded to 2dp)
        doc._.formality = round(f,2)

    # return the doc
    return doc
